import * as React from 'react';
import {observer} from "mobx-react-lite";
import {DashboardComponentModel} from "../../../../stores/DashboardComponentStore";
import {Grid, IconButton, Table, TableBody, TableCell, TableHead, TableRow} from "@material-ui/core";
import {ButtonField, CheckboxField, InputField, useReadonlyContext} from "@rx-suite/ui";
import {t, Trans} from "@lingui/macro";
import {Add, Delete} from "@material-ui/icons";
import {Annotation} from "../../../../stores/Types";
import {makeStyles} from "@material-ui/core/styles";
import {AnnotationColorPicker} from "./AnnotationColorPicker";

export type AnnotationSetterProps = {
    component: DashboardComponentModel
};

const useStyles = makeStyles(theme => ({
    danger: {
        color: theme.palette.error.main
    }
}));

export const AnnotationSetter: React.FunctionComponent<AnnotationSetterProps> = observer(({component}) => {
    const classes = useStyles();
    const readOnly = useReadonlyContext();
    const annotations = (component.viewModel.display_options?.annotations || []);
    return <>
        <Table>
            <TableHead>
                <TableRow>
                    <TableCell><Trans>Label</Trans></TableCell>
                    <TableCell><Trans>Value</Trans></TableCell>
                    {/*<TableCell><Trans>Type</Trans></TableCell>*/}
                    <TableCell><Trans>Line</Trans></TableCell>
                    <TableCell></TableCell>
                </TableRow>
            </TableHead>
            <TableBody>
                {
                    annotations.map((annotation: Annotation, i: number) => {
                        const baseName = `display_options.annotations[${i}]`;
                        const valueInputProps = component.yIsPercentage ? {min: 0, max: 100, step: 'any'} : undefined
                        return <TableRow key={annotation.id}>
                            <TableCell><InputField name={`${baseName}.label`} value={annotation.label}
                                                   onChange={component.handleChange} label={t`Label`}
                                                   disabled={readOnly}/></TableCell>
                            <TableCell><InputField name={`${baseName}.value`} value={annotation.value}
                                                   onChange={component.handleChange} label={t`Value`}
                                                   disabled={readOnly}
                                                   type={'number'}
                                                   inputProps={valueInputProps}
                            />
                            </TableCell>
                            {/*<TableCell>{annotation.type}</TableCell> note: need to handle datetimes before adding x-axis annotations*/}
                            <TableCell>
                                <Grid container alignItems={'baseline'} spacing={2}>
                                    <Grid item xs>
                                        <AnnotationColorPicker component={component} annotation={annotation}
                                                               name={`${baseName}.color`}/>
                                    </Grid>
                                    <Grid item xs><InputField name={`${baseName}.thickness`}
                                                              value={annotation.thickness}
                                                              onChange={component.handleChange} label={t`Thickness`}
                                                              disabled={readOnly}
                                                              type={'number'}
                                                              inputProps={{min: 1, max: 100, step: 1}}
                                    /></Grid>
                                    <Grid item xs><CheckboxField name={`${baseName}.dashed`}
                                                                 checked={!!annotation.dashed}
                                                                 onChange={component.handleChange} label={t`Dashed`}
                                                                 disabled={readOnly}/></Grid>
                                </Grid>
                                <div style={{
                                    width: '100%',
                                    marginTop: 4,
                                    borderTopColor: annotation.color,
                                    borderTopWidth: `${annotation.thickness}px`,
                                    borderTopStyle: annotation.dashed ? 'dashed' : 'solid',
                                }}>

                                </div>
                            </TableCell>
                            <TableCell><IconButton onClick={() => component.removeAnnotation(annotation)}><Delete
                                className={classes.danger}/></IconButton></TableCell>
                        </TableRow>
                    })}
            </TableBody>
        </Table>
        {!readOnly && <ButtonField text={t`New Annotation`} startIcon={<Add/>} disabled={readOnly}
                                   onClick={component.pushAnnotation}/>}
    </>
});