import * as React from "react";
import {observer} from "mobx-react-lite";
import {DashboardComponentTableProps} from "./DashboardComponentTable";
import {makeStyles} from "@material-ui/core/styles";
import {DashboardTargetModel} from "../../../stores/DashboardTargetStore";
import {TableColumnDescriptor} from "../../../stores/Types";
import {State} from "react-beautiful-dnd";
import {Column, ReactTableDefaults, RowInfo} from "react-table";
import classNames from "classnames";
import {ReactTableFixedColumns} from "@rx-suite/ui";
import {DashboardComponentTitle} from "../generic/DashboardComponentTitle";

const useStyles = makeStyles({
    scoreCard: {
        width: '100%',
        '& .rt-table': {
            whiteSpace: 'nowrap',
            fontSize: 'medium'
        },
        '& .rt-td': {
            textAlign: 'center',
            padding: '4px 5px'
        },
        '& .rt-tbody': {
            overflowX: 'undefined'
        },
        '& .rt-thead': {
            maxHeight: '50px'
        }
    },
    tableCell: {
        whiteSpace: 'pre-line!important' as any,
        wordWrap: 'break-word!important' as any,
    }
});

export const DashboardComponentScoreCard: React.FunctionComponent<DashboardComponentTableProps> = observer(({component}) => {
    const classes = useStyles();
    const cellProps = React.useCallback((state, rowInfo, column) => {
        const cellColor = (value: any) => {
            //Need to re-create the index used when creating the form
            const goal = component.viewModel.parameters.rows[rowInfo?.index]?.goal;
            if (!goal) return undefined;
            const goal_minimum = component.viewModel.parameters.rows[rowInfo?.index]?.goal_minimum;
            let color = undefined;
            if (typeof value !== "number") {
                if (typeof value !== "string") return undefined;
                value = parseFloat(value);
                if (isNaN(value)) return undefined;
            }
            if (value >= goal) {
                color = 'rgb(0, 100, 0, .2)';
            } else if (value >= goal_minimum) {
                color = 'rgb(231, 206, 115, .4)';
            } else {
                color = 'rgb(150, 0, 0, .2)';
            }
            return color;
        }
        return {
            style: {
                backgroundColor: cellColor(rowInfo?.row[column.id]),
            }
        }
    }, [component.viewModel.parameters.rows, classes]);

    const rowNameColProps = () => {
        return {
            style: {
                minWidth: '150px',
                textAlign: 'left'
            }
        }
    }

    const dividerCellProps = () => {
        return {
            style: {
                height: '0px',
                padding: '0px'
            }
        }
    }

    const target = component.viewModel.dashboard_target as DashboardTargetModel;
    if (!target) {
        return null;
    }
    let columns = [];
    if (component.viewModel.data && component.viewModel.data.columns) {
        columns = component.viewModel.data.columns.map((c: TableColumnDescriptor, i: number) => {
            return {
                accessor: c.id,
                Header: c.name,
                headerStyle: i === 0 ? {minWidth: '150px'} : {},
                getProps: (state: State, rowInfo: RowInfo, column: Column) => {
                    const rowParams = component.viewModel.parameters.rows[rowInfo.index];
                    const row_type = rowParams ? rowParams.row_type : undefined;
                    if (row_type === 'divider') {
                        return dividerCellProps();
                    }
                    if (i === 0) {
                        return rowNameColProps();
                    }
                    if (i === 1) {
                        return {};
                    }
                    return cellProps(state, rowInfo, column);
                },
                fixed: i === 0 ? 'left' : undefined
            }
        });
    } else if (target.returns) {
        const returns = target.returns as TableColumnDescriptor[];
        columns = returns.map(r => ({
            accessor: r.id,
            Header: r.name,
            getProps: (state: State, rowInfo: RowInfo, column: Column) => cellProps(state, rowInfo, column)
        }))
    }

    const rowProps = (rowInfo, row) => {
        if (!row) {
            return {};
        } // On a table with no rows (new model) row is undefined
        if (component.viewModel.parameters.rows[row.index]?.row_type === 'divider') {
            return {
                style: {
                    height: '0px',
                    borderBottom: 'solid 2px rgba(0,0,0,0.35)'
                }
            }
        }
        return {};
    }

    const rowGroupProps = (RowInfo, row) => {
        if (!row) {
            return {};
        }
        if (component.viewModel.parameters.rows[row.index]?.row_type === 'divider') {
            return {
                style: {
                    flex: 'unset',
                }
            }
        }
    }

    const data = component.viewModel.data ? (component.viewModel.data.data || []).slice() : [];
    return <>
        <DashboardComponentTitle component={component}/>
        <ReactTableFixedColumns data={data}
                                className={classNames(classes.scoreCard, '-striped', '-highlight')}
                                columns={columns}
                                column={
                                    {
                                        ...ReactTableDefaults.column,
                                        minWidth: 55
                                    }
                                }
                                getTrGroupProps={rowGroupProps}
                                getTrProps={rowProps}
                                sortable={false}
                                minRows={1}
                                pageSize={100}
                                showPageJump={false}
                                showPageSizeOptions={false}
                                showPagination={false}
                                resizable={false}
                                key={'scoreCard'}
        />
    </>;
});