import * as React from 'react';
import {observer} from "mobx-react-lite";
import {DashboardComponentModel} from "../../../stores/DashboardComponentStore";
import {DashboardTargetModel} from "../../../stores/DashboardTargetStore";
import {CustomTarget, TableColumnDescriptor} from "../../../stores/Types";
import ReactTable, {RowInfo, SortingRule} from "react-table";
import classNames from 'classnames';
import {makeStyles} from "@material-ui/core/styles";
import {Typography} from "@material-ui/core";
import {red, yellow} from "@material-ui/core/colors";
import CSS from "csstype";

export type DashboardComponentTableProps = {
    component: DashboardComponentModel
}

const useStyles = makeStyles(theme => ({
    table: {
        height: '100%',
        width: '100%',
    },
    tableCell: {
        whiteSpace: 'pre-line!important' as any,
        wordWrap: 'break-word!important' as any,
    },
    titleBar: {
        borderRadius: theme.spacing(1, 1, 0, 0),
        padding: theme.spacing(1, 2),
        borderWidth: 1,
        borderStyle: 'solid',
        backgroundColor: theme.palette.grey[200],
        borderColor: theme.palette.grey[300],
    },
}));

export const DashboardComponentTable: React.FunctionComponent<DashboardComponentTableProps> = observer(({component}) => {
    const classes = useStyles();
    const tdProps = React.useCallback(() => ({
        className: classes.tableCell
    }), [classes]);
    const target = component.viewModel.dashboard_target as DashboardTargetModel;
    if (!target) {
        return null;
    }
    let columns = [];
    if (component.viewModel.data && component.viewModel.data.columns) {
        columns = component.viewModel.data.columns.map((c: TableColumnDescriptor) => ({
            accessor: c.id,
            Header: c.name,
            width: target.columnDefaultWidth(c.id),
            filterable: target.isColumnFilterable(c.id),
        }));
    } else if (target.returns) {
        const returns = target.returns as TableColumnDescriptor[];
        columns = returns.map(r => ({
            accessor: r.id,
            Header: r.name,
            width: target.columnDefaultWidth(r.id),
            filterable: target.isColumnFilterable(r.id),
        }))
    }
    const getLateJobTrProps = (_: any, rowInfo: RowInfo | undefined) => {
        const props: {style?: CSS.Properties} = {};
        if (rowInfo && rowInfo.original && rowInfo.original["order-days-late"] >= 10) {
            props.style = {backgroundColor: yellow[500]};
        }
        if (rowInfo && rowInfo.original && rowInfo.original["order-days-late"] >= 15) {
            props.style = {backgroundColor: red[500]};
        }

        return props;
    };
    const trProps = target.dashboard_id === 'rx_late_list' ? getLateJobTrProps : undefined

    const hints = target.ui_hints as { default_groups?: string[], default_sorts?: (string | SortingRule)[] } || {};

    let pivots = undefined as string[] | undefined;
    const customTargets = (component.viewModel.parameters || {}).custom_targets
    if (customTargets) {
        pivots = pivots || [];
        pivots = pivots.concat(customTargets.map((t: CustomTarget) => t.id));
    }
    if (hints.default_groups) {
        pivots = pivots || [];
        pivots = pivots.concat(hints.default_groups);
    }
    const sorts = hints.default_sorts ? hints.default_sorts.map(s => {
        if (Object.prototype.hasOwnProperty.call(s, "id")) {
            return s as SortingRule;
        } else {
            return {id: s, desc: true} as SortingRule
        }
    }) : undefined;

    const title = component.viewModel.desc || component.viewModel.name;
    const data = component.viewModel.data ? (component.viewModel.data.data || []).slice() : [];
    return <>
        <div className={classes.titleBar}>
            <Typography variant={'h6'} style={component.typographyTitleStyle}>{title}</Typography>
        </div>
        <ReactTable data={data}
                    className={classNames(classes.table, '-striped', '-highlight')}
                    columns={columns}
                    pivotBy={pivots}
                    sortable={false}
                    defaultSorted={sorts}
                    getTdProps={tdProps}
                    getTrProps={trProps}
                    filterable
        />
    </>;
});
