import * as React from 'react';
import {observer} from "mobx-react-lite";
import {DashboardGroupModel, DashboardGroupStore, slideshowState} from "../../stores/DashboardGroupStore";
import {DashboardGroupDashboardModel, DashboardGroupDashboardStore} from "../../stores/DashboardGroupDashboardStore";
import {DashboardModel, DashboardStore} from "../../stores/DashboardStore";
import {DashboardGrid} from "../../components/dashboards/DashboardGrid";
import {makeStyles} from "@material-ui/core/styles";
import {Pause, PlayArrow, SkipNext, SkipPrevious} from "@material-ui/icons";
import {FullscreenButton, LinkButton} from "@rx-suite/ui";
import {IconButton, LinearProgress, Typography} from "@material-ui/core";
import {Trans} from "@lingui/macro";
import {currentUserCan} from "@rx-suite/stores";
import {DashboardComponentStore} from "../../stores/DashboardComponentStore";

const useStyles = makeStyles(theme => ({
    root: {
        height: `calc(100vh - 64px - ${theme.spacing(2)}px)`, // fullscreen, less toolbar, less toolbar margin
        "&:fullscreen": {
            height: '100vw',
            width: '100vh',
        },
        "&:hover $toolbarWrapper, & .Mui-focused $toolbarWrapper": {
            opacity: 1
        },
        position: 'relative'
    },
    fullscreenTarget: {
        height: '100%',
        width: '100%',
        "&:fullscreen": {
            height: '100vw',
            width: '100vh',
        }
    },
    toolbarWrapper: {
        position: 'absolute',
        opacity: 0,
        transition: 'opacity 0.25s',
        top: 0,
        left: 0,
        width: '100%',
        zIndex: 99999
    },
    toolbarButtons: {
        float: 'right'
    }
}));

export const DashboardGroupSlideshow: React.FunctionComponent<{ groupId?: string, initialMemberId?: string }> = observer(({
                                                                                                                              groupId,
                                                                                                                              initialMemberId
                                                                                                                          }) => {
    const classes = useStyles();
    const fullscreenLayoutRef = React.createRef<HTMLDivElement>();
    const member = DashboardGroupDashboardStore.data.get(slideshowState.memberId?.toString() || '');
    const dashboard = DashboardStore.data.get(member?.dashboard_id?.toString() || '');
    let group: DashboardGroupModel | undefined = undefined;
    if (groupId) {
        group = DashboardGroupStore.get(groupId);
    } else {
        group = DashboardGroupStore.models[0] as DashboardGroupModel;
    }
    const [hasLoadedOnce, setHasLoadedOnce] = React.useState(false);
    const [hasInitialized, setHasInitialized] = React.useState(false);
    React.useEffect(() => {
        DashboardGroupStore.getData().then(() => {
            DashboardGroupDashboardStore.getData().then(() => {
                DashboardStore.getData().then(() => {
                    setHasLoadedOnce(true);
                });
            });
        });
    }, [setHasLoadedOnce]);
    React.useEffect(() => {
        if (hasLoadedOnce && !hasInitialized) {
            let member: DashboardGroupDashboardModel | undefined = undefined;
            if (initialMemberId) {
                member = DashboardGroupDashboardStore.get(initialMemberId);
            }
            if (!member && group) {
                member = group.members[0];
            }
            if (member) {
                setHasInitialized(true);
                slideshowState.play(member);
            }
        }
    }, [slideshowState, hasLoadedOnce, initialMemberId, group, hasInitialized, setHasInitialized]);
    const handlePlayPause = React.useCallback(() => {
        if (slideshowState.playing) {
            slideshowState.pause();
        } else {
            let member: DashboardGroupDashboardModel | undefined = undefined;
            if (slideshowState.memberId) {
                member = DashboardGroupDashboardStore.get(slideshowState.memberId);
            }
            if (!member && initialMemberId) {
                member = DashboardGroupDashboardStore.get(initialMemberId);
            }
            if (!member && group) {
                member = group.members[0];
            }
            if (member) {
                slideshowState.play(member);
            }
        }
    }, [slideshowState]);

    const permissionsAccepted = currentUserCan(DashboardComponentStore.permissionTarget!, 'read') &&
        currentUserCan(DashboardStore.permissionTarget!, 'read') &&
        currentUserCan(DashboardGroupStore.permissionTarget!, 'read');

    if (!permissionsAccepted) {
        return <Typography>
            <Trans>You must have the Read permission for Dashboard Components, Dashboards, and Dashboard Groups to view
                the Slideshow</Trans>
        </Typography>
    }

    if (!dashboard) {
        if (DashboardStore.isLoading) {
            return <LinearProgress/>
        } else {
            if (currentUserCan(DashboardGroupStore.permissionTarget!, 'create')){
                return <LinkButton to={DashboardGroupStore.listLink()!}>
                    <Trans>Setup Dashboard Groups</Trans>
                </LinkButton>;
            } else {
                return <Typography>
                    <Trans>Ask an administrator to set up Dashboard Groups</Trans>
                </Typography>;
            }
        }
    }
    return (
        <div className={classes.root}>
            <div className={classes.toolbarWrapper}>
                <div className={classes.toolbarButtons}>
                    <FullscreenButton fullscreenRef={fullscreenLayoutRef}/>
                    <IconButton onClick={slideshowState.goBack}>
                        <SkipPrevious/>
                    </IconButton>
                    <IconButton onClick={handlePlayPause}>
                        {slideshowState.playing ? <Pause/> : <PlayArrow/>}
                    </IconButton>
                    <IconButton onClick={slideshowState.goNext}>
                        <SkipNext/>
                    </IconButton>
                </div>
            </div>
            <div className={classes.fullscreenTarget} ref={fullscreenLayoutRef}>
                <DashboardGrid dashboard={dashboard! as DashboardModel}/>
            </div>
        </div>
    )
});
