import {CanvasRenderer} from "./CanvasRenderer";
import {SystemSettingStore} from "@rx-suite/stores";
import {LabLayoutComponentModel, LayoutComponentPosition} from "../LabLayoutComponentStore";
import {t} from "@lingui/macro";

type GroupedColorData = {
    [color: string]: Array<string>;
};

export class LayoutDashboardViewRenderer extends CanvasRenderer {
    constructor() {
        super();

        setInterval(() => {
            this.drawAlarms(); // SIDE-EFFECT: draws the circle above all other components [OK for now]
            this.drawSelectedComponent();
        }, 1000)
    }

    colors: undefined | ColorData;
    machines: undefined | MachineData;
    lastRunTime = "none";

    alarmComponents: LabLayoutComponentModel[] = [];
    alarmBlink = true;

    public setData(data: StatusData, lastRuntime: string): void {
        const machines: MachineData = data.machines;
        if (machines) this.machines = machines;

        const colors: ColorData = data.colors;
        if (colors) this.colors = colors;

        const systemSettings = SystemSettingStore.selectedModel;
        this.lastRunTime = systemSettings ? systemSettings.printDateTime(new Date(lastRuntime)) : lastRuntime;

        this.detectAlarms();
        this.draw();
    }

    protected afterSetLayout(): void {
        this.detectAlarms();
    }

    protected handleDraw(): void {
        // Use the view model to render the positions
        this.layout.current_ordered_component.forEach(c => {
            switch (c.component_type) {
                case "machine_status_dot":
                    this.drawDetailedMachineDot(c);
                    break;
                case "image":
                    this.drawImage(c);
                    break;
                case "alert_group":
                    //Do not draw alert groups in normal renders, they will be drawn via the alerts
                    break;
                case "status_legend":
                    this.drawStatusLegend(c);
                    break;
            }
        });
        this.drawAlarms(false);
        this.drawSelectedComponent()
        this.drawText(`Last Run: ${this.lastRunTime}`, {x: 10, y: this.canvas.height - 15});
    }

    private drawSelectedComponent(): void {
        if (!this.selectedComponent) return;
        const c = this.selectedComponent;

        let data;
        if (this.machines) {
            data = this.machines[c.machine_sn]
        }

        const boxHeight = 90;
        const xPos = c.position.x - 50 > 0 ? c.position.x - 50 : 0;
        const yPos = c.position.y - boxHeight - 15 > 0 ? c.position.y - boxHeight - 15 : 0;
        const boxBasePos = {x: xPos, y: yPos};
        const lMargin = 5;
        const lineSpacing = 20;

        this.drawRectangle(boxBasePos, 250, boxHeight, {fillColor: 'white', strokeColor: 'black', lineWidth: 2})

        if (!data) {
            this.drawText(t`No Data Available`, {x: boxBasePos.x + lMargin, y: boxBasePos.y + (1 * lineSpacing)});
        } else {
            const time = SystemSettingStore.selectedModel ? SystemSettingStore.selectedModel.printDateTime(new Date(data.time)) : data.time;
            this.drawText(c.name, {x: boxBasePos.x + lMargin, y: boxBasePos.y + (1 * lineSpacing)});
            this.drawText(t`status:` + ' ' + data.status.toUpperCase(), {
                x: boxBasePos.x + (2 * lMargin),
                y: boxBasePos.y + (2 * lineSpacing)
            });
            this.drawText(t`duration:` + ' ' + LayoutDashboardViewRenderer.statusDurationToString(data.duration).toUpperCase(), {
                x: boxBasePos.x + (2 * lMargin),
                y: boxBasePos.y + (3 * lineSpacing)
            });
            this.drawText(t`last update:` + ' ' + time, {
                x: boxBasePos.x + (2 * lMargin),
                y: boxBasePos.y + (4 * lineSpacing)
            });
        }
    }

    private drawDetailedMachineDot(c: LabLayoutComponentModel): void {
        const status = this.getMachineStatus(c.machine_sn);
        const color = this.getStatusColor(status);
        const duration = this.getMachineDuration(c.machine_sn);
        this.drawMachineInfo(c, c.name, LayoutDashboardViewRenderer.shortenedStatusDurationToString(duration), color);
    }

    private drawAlarms(nextBlinkCycle = true): void {
        if (!this.ready) return;
        for (const c of this.alarmComponents) {
            // Note: make the group alerts alternate from the machine alerts to give more contrast
            if (c.component_type === 'machine_status_dot') {
                this.drawCircle({
                    x: c.x,
                    y: c.y
                }, c.width, this.alarmBlink ? {fillColor: CanvasRenderer.COLOR_ALERT_ON} : {fillColor: CanvasRenderer.COLOR_ALERT_OFF});
            } else if (c.component_type === 'alert_group') {
                this.drawAlertGroup(c, !this.alarmBlink ? {fillColor: CanvasRenderer.COLOR_ALERT_ON} : {fillColor: CanvasRenderer.COLOR_ALERT_OFF})
            }
        }
        // To stop jitters from happening from successive quick draws, allow draw cycle to skip toggle of blink
        if (nextBlinkCycle) this.alarmBlink = !this.alarmBlink;
    }

    private detectAlarms(): void {
        this.alarmComponents = [];
        if (!this.machines || !this._layout) return;

        const groups: any = {};

        // Individual machine alarms
        for (const sn in this.machines) {
            const data = this.machines[sn];

            if (!data.status) continue;
            const status = data.status.toUpperCase();

            // add status to group counts
            for (const g of data.groups) {
                if (!groups[g]) {
                    groups[g] = []
                }
                groups[g].push(status === "ERROR" ? 1 : 0);
            }

            if (status !== "ERROR") continue;

            const component = this.layout.components.find(c => c.machine_sn === sn);
            if (component && (data.duration >= component.alert_after)) {
                this.alarmComponents.push(component)
            }
        }

        // Find if any group needs to go into alert
        for (const g in groups) {
            if (groups[g].every(v => v === 1)) {
                const alertGroup = this.layout.components.find(c => c.machine_group_ids.includes(Number(g)));
                if (!alertGroup) continue;

                // A previous group in alert may already make the alert component appear
                if (!this.alarmComponents.includes(alertGroup)) {
                    this.alarmComponents.push(alertGroup)
                }
            }
        }
    }

    private drawStatusLegend(c: LabLayoutComponentModel): void {
        if (!c.viewModel.options || !JSON.parse(c.viewModel.options).show) {
            return; // do not draw
        }

        const scalingRatio = this.scaleRatio;

        const colorSquareSize = (20 * scalingRatio.x);

        // HORIZONTAL SPACING
        const horizontalPadding = (10 * scalingRatio.x);
        const insetPadding = (20 * scalingRatio.x);
        const statusTextMargin = (10 * scalingRatio.x);
        const characterWidth = (8 * scalingRatio.x);

        // VERTICAL SPACING
        const verticalTitleMargin = (20 * scalingRatio.y);
        const verticalSpacing = ((colorSquareSize + 15) * scalingRatio.y);
        const verticalPadding = (30 * scalingRatio.y);
        const characterHeight = (7 * scalingRatio.y);

        const colors = this.getGroupedColors();

        const statusAmount = Object.keys(colors).length;
        const totalHeight = verticalPadding + verticalTitleMargin + (statusAmount * verticalSpacing);

        const longestStatusString = Object.values(colors).map(o => o.join('/')).reduce((a, b) => a.length > b.length ? a : b, '');
        const totalWidth = horizontalPadding + insetPadding + colorSquareSize + statusTextMargin + (longestStatusString.length * characterWidth) + horizontalPadding;

        this.drawRectangle(c.position, totalWidth, totalHeight, { fillColor: 'rgba(255, 255, 255, 0.5)', strokeColor: 'black', lineWidth: 2 });
        this.drawStatusLegendTitleText(c.name, {x: c.position.x + horizontalPadding, y: c.position.y + verticalPadding});

        let spacingIndex = 0;

        for (const colorKey in colors) {

            const statusArr = colors[colorKey];

            this.drawRectangle({
                x: c.position.x + horizontalPadding + insetPadding,
                y: c.position.y + verticalPadding + verticalTitleMargin + (verticalSpacing * spacingIndex)
            },  colorSquareSize, colorSquareSize, { fillColor: colorKey, strokeColor: 'black' });

            this.drawColorText(statusArr.join('/'), {
                x: c.position.x + horizontalPadding + insetPadding + colorSquareSize + statusTextMargin,
                y: c.position.y + verticalPadding + verticalTitleMargin + (colorSquareSize / 2) + (characterHeight / 2) + (verticalSpacing * spacingIndex)
            });

            spacingIndex++;
        }
    }

    private drawColorText(text: string, pos: LayoutComponentPosition): void {
        const scalingRatio = this.scaleRatio;
        const fontSize = (13 * scalingRatio.x) + 'px';
        this.ctx.font = '600 ' + fontSize + ' Segoe UI';
        this.ctx.fillStyle = "black";
        this.ctx.textAlign = "start";
        this.ctx.fillText(text, pos.x, pos.y);
    }

    private getMachineDuration(sn: string): number {
        if (this.machines) {
            const machine = this.machines[sn];
            if (machine) return machine.duration;
        }
        return -1;
    }

    private getMachineStatus(sn: string): string {
        if (this.machines) {
            const machine = this.machines[sn];
            if (machine) return machine.status;
        }
        return "NOTCONNECTED";
    }

    private getStatusColor(status: string): string {
        if (status) {
            status = status.toUpperCase();
            let color;
            if (this.colors) {
                color = this.colors[status];
            } else {
                color = LayoutDashboardViewRenderer.defaultColors[status];
            }
            if (color) return color;
        }
        return "#808080";
    }

    private static statusDurationToString(duration: number): string {
        if (duration < 0) {
            return t`N/A` // State is in the future, duration makes no sense
        } else if (duration < 60) {
            return t`less than a minute`
        } else if (duration < 3600) {
            const minutes = Math.floor(duration / 60);
            return minutes === 1 ? t`1 minute` : (`${minutes}` + ' ' + t`minutes`)
        } else if (duration < 86400) {
            const hours = Math.floor(duration / 3600);
            return hours === 1 ? t`1 hour` : (`${hours}` + ' ' + t`hours`)
        } else {
            return t`more than a day`
        }
    }

    private static shortenedStatusDurationToString(duration: number): string {
        if (duration < 0) {
            return t`N/A` // State is in the future, duration makes no sense
        } else if (duration < 60) {
            return t`< minute`
        } else if (duration < 3600) {
            const minutes = Math.floor(duration / 60);
            return minutes === 1 ? t`1 minute` : (`${minutes}` + ' ' + t`minutes`)
        } else if (duration < 86400) {
            const hours = Math.floor(duration / 3600);
            return hours === 1 ? t`1 hour` : (`${hours}` + ' ' + t`hours`)
        } else {
            return t`> day`
        }
    }

    private getGroupedColors(): GroupedColorData {
        const colors: ColorData = this.colors ? this.colors : LayoutDashboardViewRenderer.defaultColors;

        const groupedColors: GroupedColorData = {};

        for (const statusKey in colors) {
            const color = colors[statusKey];

            if (!(color in groupedColors)) {
                groupedColors[color] = [];
            }

            groupedColors[color].push(statusKey);
        }

        return groupedColors;
    }

    private static get defaultColors(): ColorData {
        return {
            "NOTCONNECTED": "#808080",
            "CALIBRATION": "#ff5000",
            "MAINTENANCE": "#00c4ff",
            "OFF": "#000000",
            "ESTOP": "#ff0080",
            "HALT": "#ffa0a0",
            "ON": "#ffffc4",
            "SETUP": "#ffff00",
            "NOTREADY": "#ffa000",
            "READY": "#0000ff",
            "ERROR": "#ff0000",
            "WAITINGFORJOBS": "#00ff00",
            "PROCESSING": "#008000"
        }
    }

    //// Canvas Action Handler

    public handleMouseMove(e: any): void {
        if (e.touches) e.preventDefault();
        const point = this.eventInputPosition(e);
        const comp = this.layout.current_ordered_component.filter(c => c.component_type === 'machine_status_dot').find(c => c.pointOverlapsComponent(point));
        if (comp) {
            if (!this.selectedComponent) {
                this.setSelectedComponent(comp);
            }
        } else {
            if (this.selectedComponent) {
                this.setSelectedComponent(undefined);
            }
        }
    }
}

type ColorData = {
    [color: string]: string
}

type MachineData = {
    [sn: string]: {
        status: string;
        time: string;
        duration: number;
        groups: number[];
    }
}

type StatusData = {
    machines: MachineData,
    colors: ColorData
}
